<style>
html {
    background: white;
    font-family: serif;
    margin-left: 3em;
    margin-right: 2em;
    margin-top: 2em;
}

form#searchform {
    font-family: monospace;
    text-align: right;
}

div.actions {
    font-family: monospace;
    text-align: right;
}

div.actions ul, div.actions li {
    display: inline;
}

div.pageheader {
    font-family: monospace;
    margin-bottom: 2em;
}

div.pageheader span.title {
    display: block;
    font-size: 200%;
    font-weight: bold;
    font-family: sans-serif;
    margin-top: 0.5em;
}

div#pagebody {
}

div.pagefooter {
    font-family: monospace;
    margin-top: 3em;
}

div#pagebody {
}

div#TOC ul {
    list-style: none;
}

h1, h2, h3, h4, h5, h6 {
    font-family: sans-serif;
    font-weight: bold;
    margin-top: 2em;
}

h1 {
    font-size: 150%;
}

h2 {
    font-size: 120%;
}

h3 {
    font-size: 100%;
}

ul li, ol li {
    margin-top: 0.5em;
    margin-bottom: 0.5em;
}

pre {
    margin-left: 4em;
}

</style>
